#pragma once
#include "define.h"
#include "glm.h"

#define  DEFAULTUPDATE	20
#define  DEFAULTSPEED	2

class CObject3D
{
private:
	GLMmodel *m_pModel;

	Point3f	 m_vPosition;

	float   m_fLRTheata;

	float	m_fMaxSpeed;
	float	m_fMinSpeed;
	float	m_fMoveSpeed;
	float	m_fHalfSpeed;

	DWORD   m_dSpeedOldTick;
	DWORD	m_dMovetick;
	DWORD	m_dMoveTerm;

public:
	CObject3D(char *);
	~CObject3D(void);

	void leftRotate(); 
	void rightRotate();

	void MoveObject();
	void calSpeed(bool);
	void RenderObject();

	inline Point3f getPos() { return m_vPosition; }
	inline float   getRotate() { return m_fLRTheata; }
};
